import asyncio
from asyncio import run
from typing import List
from .abstract_auth import AbstractAuth

from .air_conditioner import AirConditioner


class HomecomApi:
    """Class to communicate with the HomeCom API."""

    def __init__(self, auth: AbstractAuth):
        """Initialize the API and store the auth so we can make requests."""
        self.auth = auth

    async def async_get_acs(self) -> List[AirConditioner]:
        resp = await self.auth.request("get", "pointt-api/api/v1/gateways")
        print(await resp.text())
        resp.raise_for_status()
        acs = filter(lambda x: x["deviceType"] == "rac", await resp.json())
        return await asyncio.gather(*[self.async_get_ac(ac['deviceId']) for ac in acs])

    async def async_get_ac(self, device_id: int) -> AirConditioner:
        resp = await self.auth.request("get", f"pointt-api/api/v1/gateways/{device_id}/resource/airConditioning"
                                              "/standardFunctions")
        print(await resp.text())
        resp.raise_for_status()
        return AirConditioner(device_id, await resp.json(), self.auth)



from enum import Flag, IntFlag
from strenum import StrEnum
from .abstract_auth import AbstractAuth


class AcControl(IntFlag):
    ON = 0
    OFF = 1

    @classmethod
    def from_str(cls, input: str):
        return cls(int(input == "off"))

    def to_str(self):
        if self == AcControl.ON:
            return "on"
        return "off"


class OperationMode(StrEnum):
    AUTO = "auto"
    DRY = "dry"
    HEAT = "heat"
    COOL = "cool"


class AirConditioner:
    """Class that represents a AirConditioner object in the HomeCom API."""

    def __init__(self, device_id: int, raw_data: dict, auth: AbstractAuth):
        """Initialize a air conditioner object."""
        self.device_id = device_id
        self.raw_data = raw_data
        self.auth = auth

    # Note: each property name maps the name in the returned data

    @property
    def id(self) -> int:
        """Return the ID of the AC."""
        return self.device_id

    @property
    def is_on(self) -> AcControl:
        """Return if the light is on."""
        return AcControl.from_str(self.raw_data["references"][1]["value"])

    async def async_set_temperature(self, temperature: float):
        """Set the target temperature."""
        resp = await self.auth.request(
            "put",
            f"pointt-api/api/v1/gateways/{self.id}/resource/airConditioning/temperatureSetpoint",
            json={"value": temperature},
        )
        print(await resp.text())
        resp.raise_for_status()

    async def async_set_operation_mode(self, operation_mode: OperationMode):
        """Set the operation mode, like `heat`, `cool`, etc."""
        resp = await self.auth.request(
            "put",
            f"pointt-api/api/v1/gateways/{self.id}/resource/airConditioning/operationMode",
            json={"value": operation_mode},
        )
        print(await resp.text())
        resp.raise_for_status()

    async def async_set_ac_control(self, ac_control: AcControl):
        """Turn the AC on/off."""
        resp = await self.auth.request(
            "put",
            f"pointt-api/api/v1/gateways/{self.id}/resource/airConditioning/acControl",
            json={"value": ac_control.to_str()},
        )
        print(await resp.text())
        resp.raise_for_status()



import base64
import hashlib
import logging
import os
import re
from urllib import parse
from datetime import datetime, timedelta

import aiohttp
from aiohttp import ClientSession, BasicAuth
from lxml import html

from .abstract_auth import AbstractAuth

logging.basicConfig(level=os.environ.get("LOGLEVEL", "DEBUG"))
log = logging.getLogger(__name__)

# Constants
CLIENT_ID = "app.dashtt.pointt.android"
CLIENT_SECRET = "QuKV5lWbLIkZMX65IEzxowqTtp1oD7srDiMwrFuz"
BASE_URL = "https://accounts.ttsignon.com/"
AUTHORIZE_URL = "https://accounts.ttsignon.com/connect/authorize"
ACCESS_TOKEN_URL = "https://accounts.ttsignon.com/connect/token"


class HomecomAuth(AbstractAuth):
    def __init__(self, websession: ClientSession, host: str, auth: BasicAuth):
        """Initialize the auth."""
        super().__init__(websession, host)

        self.auth = auth
        self.expires_in = None
        self.refresh_token = None
        self.access_token = None
        self.issued_at = None

    def is_valid(self) -> bool:
        return (self.issued_at + timedelta(seconds=self.expires_in)) > datetime.utcnow()

    async def async_get_access_token(self) -> str:
        """Return a valid access token."""
        if not self.access_token:
            await self.authenticate()
        if not self.is_valid():
            await self.refresh_access_token()
        if self.access_token:
            return self.access_token
        raise NotAuthenticated()

    async def authenticate(self):
        code_challenge, code_verifier = generate_code_challenge()
        authorize_url = (
            AUTHORIZE_URL
            + "?client_id=app.dashtt.pointt.android&response_type=code&scope=openid%20offline_access"
              "%20user.identity%20user.write%20user.read%20company.identity%20company.read%20tos.write"
              "%20pointt.gateway.claiming%20pointt.gateway.removal%20pointt.gateway.list%20pointt"
              ".gateway.users%20pointt.gateway.resource.dashapp&redirect_uri=com.bosch.tt.dashtt"
              ".pointt%3A%2F&code_challenge_method=S256"
              f"&code_challenge={code_challenge}"
        )
        o_auth_session = aiohttp.ClientSession()
        async with o_auth_session as session:
            log.debug(authorize_url)
            response = await session.get(authorize_url, allow_redirects=True)
            print(response.headers)

            assert response.status == 200
            tree = html.fromstring(await response.text())
            return_url = tree.xpath("//*[@name='ReturnUrl']")[0].get("value")
            verification_token = tree.xpath("//*[@name='__RequestVerificationToken']")[
                0
            ].get("value")
            log.debug(verification_token)
            log.debug(return_url)
            response = await session.post(
                "https://identity-myprofile.bosch.com" + "/ids/login",
                data={
                    "uEmail": self.auth.login,
                    "uPassword": self.auth.password,
                    "ReturnUrl": return_url,
                    "__RequestVerificationToken": verification_token,
                },
                allow_redirects=False,
            )
            log.debug(await response.text())
            log.debug(response.status)
            log.debug(response.headers)
            assert response.status == 302
            location = response.headers["Location"]
            cookie = response.headers["Set-Cookie"]
            response = await session.get(
                "https://identity-myprofile.bosch.com" + location,
                headers={"Set-Cookie": cookie},
                allow_redirects=False,
            )
            location = response.headers["Location"]
            log.debug(location)
            log.debug(response.status)

            cookie = response.headers["Set-Cookie"]
            response = await session.get(
                location, headers={"Set-Cookie": cookie}, allow_redirects=False
            )

            location = response.headers["Location"]
            log.debug(location)
            log.debug(response.status)

            response = await session.get(
                "https://identity.bosch.com/" + location,
                headers={"Set-Cookie": cookie},
                allow_redirects=False,
            )

            location = response.headers["Location"]
            log.debug(location)
            log.debug(response.status)

            response = await session.get(
                location, headers={"Set-Cookie": cookie}, allow_redirects=False
            )
            log.debug(response.headers)
            log.debug(response.status)

            tree = html.fromstring(await response.text())
            code = tree.xpath("//*[@name='code']")[0].get("value")
            state = tree.xpath("//*[@name='state']")[0].get("value")
            session_state = tree.xpath("//*[@name='session_state']")[0].get("value")

            response = await session.post(
                BASE_URL + "signin-oidc",
                data={"code": code, "state": state, "session_state": session_state},
                allow_redirects=False,
            )
            location = response.headers["Location"]
            log.debug(location)
            log.debug(response.headers)
            log.debug(response.status)

            response = await session.get(
                BASE_URL + location,
                headers={"Set-Cookie": cookie},
                allow_redirects=False,
            )
            log.debug(response.headers)
            location = response.headers["Location"]
            log.debug(response.status)

            query = parse.urlparse(location).query
            redirect_params = parse.parse_qs(query)
            log.debug(redirect_params)
            code = redirect_params["code"][0]
            session_state = redirect_params["session_state"][0]
            scope = redirect_params["scope"][0]
            log.debug(code)

            data = {
                "grant_type": "authorization_code",
                "code": code,
                "client_id": CLIENT_ID,
                "client_secret": CLIENT_SECRET,
                "redirect_uri": "com.bosch.tt.dashtt.pointt:/",
                "code_verifier": code_verifier,
                "session_state": session_state,
                "scope": scope,
            }

            response = await session.post(
                ACCESS_TOKEN_URL,
                data=data,
            )
            log.debug(response.headers)
            token = await response.json()
            assert response.status == 200
            self.refresh_token = token["refresh_token"]
            self.access_token = token["access_token"]
            self.expires_in = token["expires_in"]
            self.issued_at = datetime.utcnow()

    async def refresh_access_token(self):
        o_auth_session = aiohttp.ClientSession()
        async with o_auth_session as session:
            data = {
                "grant_type": "refresh_token",
                "refresh_token": self.refresh_token,
                "client_id": CLIENT_ID,
                "client_secret": CLIENT_SECRET,
            }
            response = await session.post(
                ACCESS_TOKEN_URL,
                headers={"Content-Type": "application/x-www-form-urlencoded"},
                data=data,
            )
            token = await response.json()
            assert response.status == 200
            self.access_token = token["access_token"]
            self.refresh_token = token["refresh_token"]
            self.expires_in = token["expires_in"]
            self.issued_at = datetime.utcnow()
            await session.close()


def generate_code_challenge():
    code_verifier = base64.urlsafe_b64encode(os.urandom(40)).decode("utf-8")
    code_verifier = re.sub("[^a-zA-Z0-9]+", "", code_verifier)

    code_challenge = hashlib.sha256(code_verifier.encode("utf-8")).digest()
    code_challenge = base64.urlsafe_b64encode(code_challenge).decode("utf-8")
    return code_challenge.replace("=", ""), code_verifier


class NotAuthenticated(Exception):
    """User not authenticated."""



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['homecom']

package_data = \
{'': ['*']}

install_requires = \
['StrEnum>=0.4.8,<0.5.0',
 'aiohttp>=3.8.1,<4.0.0',
 'lxml>=4.9.1,<5.0.0',
 'pyppeteer>=1.0.2,<2.0.0',
 'python-dotenv>=0.21.0,<0.22.0']

setup_kwargs = {
    'name': 'homecom',
    'version': '0.1.3',
    'description': '',
    'long_description': 'None',
    'author': 'neugartf',
    'author_email': 'fabianneugart@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
